#########################################################################
# Secret
#
# Author:    Fabio Santos B. da Silva
# Email :    fsbsilva@gmail.com
# Version:   0.1.0
# Date:      06/02/2018
# Purpose:   It is going to encrypt and decrypt a plain_text
#########################################################################

require 'puppet_x/secret/secret'
require 'puppet/face'

Puppet::Face.define(:secret, '0.1.0') do
  copyright "Fabio Santos B. da Silva", 2018
  license 'Apache-2.0'

  summary "It is going to encrypt and decrypt a plain_text"
  description <<-EOT
    It is simple module to decrypt and encrypt a plain_text in order to be re-used
    within many custom function modules. It will be a complement in your solution.

    If you are developing a solution to install an software that need to have a
    secret information, you can create a function where it can read a particular
    facter, hiera for file that contains a secret_key in order to protect that information.
  EOT

  action :encrypt do
    summary 'This action is going to encrypt a plain_text'
    arguments "<plain_text>"
    description <<-EOT
      That action is going to use a default secret_key and iv whether you do not
      have one.

      Examples:
      puppet secret ecrypt "password123"

      puppet secret encrypt --secretkey '41254157g5o9s7h8m3e612awpmon!5jk' --iv_base 'nxWo3eFMpPxxY+sbade4eg==' password123

    EOT

    option '--secretkey ' do
      summary 'This action is going to use a custom secretkey to encrypt a plain_text'
      default_to { false }
    end

    option '--iv_base64 ' do
      summary 'This action is going to use a custom iv_base64 to encrypt a plain_text'
      default_to { false }
    end

    when_invoked do |plain_text, options|
        secretkey = options[:secretkey] || PuppetX::Secret::Secret::DEFAULT_KEY
        iv_base64 = options[:iv_base64] || PuppetX::Secret::Secret::DEFAULT_IV

        puts PuppetX::Secret::Secret.encrypt( plain_text, secretkey, iv_base64)
    end
  end

  action :decrypt do
    summary 'This action is going to decrypt a crypted_base64'
    arguments "<crypted_base64>"
    description <<-EOT
      That action is going to use a default secret_key and iv whether you do not
      have one.

      Examples:
      puppet secret decrypt 't5LmQf9tEKCqVR0AkqNWGw=='

      puppet secret decrypt --secretkey '41254157g5o9s7h8m3e612awpmon!5jk' --iv_base 'nxWo3eFMpPxxY+sbade4eg==' 't5LmQf9tEKCqVR0AkqNWGw=='

    EOT

    option '--secretkey ' do
      summary 'This action is going to use a custom secretkey to decrypt a crypted_base64'
      default_to { false }
    end

    option '--iv_base64 ' do
      summary 'This action is going to use a custom iv_base64 to decrypt a crypted_base64'
      default_to { false }
    end

    when_invoked do |crypted_base64, options|
        secretkey = options[:secretkey] || PuppetX::Secret::Secret::DEFAULT_KEY
        iv_base64 = options[:iv_base64] || PuppetX::Secret::Secret::DEFAULT_IV

        decrypted = PuppetX::Secret::Secret.decrypt( crypted_base64, secretkey, iv_base64)
        puts "decrypted: #{decrypted}\n"
    end
  end

  action :random_iv do
    summary 'This action is going to generate an iv_base64'
    description <<-EOT
      That action is going to generate a iv_base64 if you do not have one.

      Example:
      puppet secret random_iv

    EOT

    when_invoked do |*args|
        iv_base64 = PuppetX::Secret::Secret.random_iv()
        puts "iv_base64: #{iv_base64}"
    end
  end

end
